#!/bin/sh

# script to detect new dhcp lease

# this will be called by dnsmasq everytime a new device is connected
# with the following arguments
# $1 = add | old | del
# $2 = mac address
# $3 = ip address
# $4 = device name

[ "$1" == del ] && exit 0

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

# bug on 501
model="`$SCRIPT/model.sh`"
[ $model == 501 ] && exit 0

source $CONFIG/ap.conf
source $CONFIG/ng.conf

source $SCRIPT/utils/hotspot.sh
source $SCRIPT/utils/wifi.sh

debug() {
  [ ! -z "$DEBUG" ] && echo $1 >> /var/run/hotspot-autologin.log
}

hotspots=`getHotspotInstanceList`

ip=$3
ssid=0
inf="`cat /proc/net/arp | grep \"$3 \" | awk '{print $6}'`"

for s in $hotspots ; do
  hotspotInf="`getSsidInf $s`"
  if [ "$inf" == "$hotspotInf" ] ; then
    ssid=$s
    break
  fi
done

[ "$ssid" -eq 0 ] && exit 0

debug "$1 ip $3 ($2) connected to ssid $ssid"

eval autoLogin="\$SSID${ssid}_AUTO_LOGIN"
if [ "$autoLogin" != 1 ] ; then
  debug "auto login dsabled"
  exit 0
fi

ipset test "loginlist${ssid}" $ip &>/dev/null
if [ $? -eq 0 ] ; then
  debug "logged in"
  exit 0
fi

ipset add "autologin${ssid}" $ip timeout 10
