#!/bin/sh

banner() {
	echo "┌─┐┌─┬───┬───┬───┬─┐░┌┬───┬────┐"
	echo "││└┘││┌──┤┌─┐│┌─┐││└┐││┌──┤┌┐┌┐│"
	echo "│┌┐┌┐│└──┤│░└┤│░││┌┐└┘│└──┼┘││└┘"
	echo "││││││┌──┤│┌─┤└─┘││└┐││┌──┘░││░░"
	echo "││││││└──┤└┴─│┌─┐││░│││└──┐░││░░"
	echo "└┘└┘└┴───┴───┴┘░└┴┘░└─┴───┘░└┘░░"
	echo "================================"
	echo
}


readData() {
	eval "res=$3"

	echo -n "$1 [$res]: "
	read res
	if [ -z "$res" ] ; then
		eval "res=$3"
	fi

	eval "$2=$res"
}

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script
TYPE=$ROOT/models
DATA=/data

# Init API config
if [ "$skip" != true ] ; then
	banner

	API_CONFIG="$CONFIG/api.conf"
	[ -f "$API_CONFIG" ] && source "$API_CONFIG"

	readData "Cloud address" CLOUD "$CLOUD"
	readData "Cloud port" PORT "$PORT"
	readData "Cloud scheme" SCHEME "$SCHEME"

	CLOUD_HOST="$CLOUD"
	[ "$SCHEME" == http -a "$PORT" != 80 ] && CLOUD_HOST="$CLOUD:$PORT"
	[ "$SCHEME" == https -a "$PORT" != 443 ] && CLOUD_HOST="$CLOUD:$PORT"

	CLOUD_ADDRESS="$SCHEME://$CLOUD_HOST/cloud/api/config"
	readData "Config API address" CLOUD_ADDRESS "$CLOUD_ADDRESS"

	AUTH_ADDRESS="$SCHEME://$CLOUD_HOST/cloud/api/session"
	readData "Auth API address" AUTH_ADDRESS "$AUTH_ADDRESS"

	readData "Device ID" DEVICE_ID "$DEVICE_ID"

	echo -n > "$API_CONFIG"
	echo "CLOUD='$CLOUD'" >> "$API_CONFIG"
	echo "CLOUD_ADDRESS='$CLOUD_ADDRESS'" >> "$API_CONFIG"
	echo "AUTH_ADDRESS='$AUTH_ADDRESS'" >> "$API_CONFIG"
	echo "DEVICE_ID='$DEVICE_ID'" >> "$API_CONFIG"

	# Init MQT config

	MQTT_CONFIG="$CONFIG/mqtt.conf"
	[ -f "$MQTT_CONFIG" ] && source "$MQTT_CONFIG"

	readData "MQTT address" MQTT_ADDR "$bishbosh_server"
	readData "MQTT port" MQTT_PORT "$bishbosh_port"
	readData "MQTT username" MQTT_USER "$bishbosh_connect_username"
	readData "MQTT password" MQTT_PASS "$bishbosh_connect_password"

	echo -n > "$MQTT_CONFIG"
	echo "bishbosh_server='$MQTT_ADDR'" >> "$MQTT_CONFIG"
	echo "bishbosh_port='$MQTT_PORT'" >> "$MQTT_CONFIG"
	echo "bishbosh_connect_username='$MQTT_USER'" >> "$MQTT_CONFIG"
	echo "bishbosh_connect_password='$MQTT_PASS'" >> "$MQTT_CONFIG"

fi

# Init model
sh $TYPE/init.sh

echo "  Install dependencies"
$SCRIPT/setup/setup-opkg.sh

read -r -d '' packages << EOM
ca-certificates
wget
luasec
mosquitto-client-ssl
netcat
privoxy
ipset-dns
libopenssl
openssl-util
tcpdump
hostapd-utils
EOM
if opkg update ; then
	for p in $packages ; do
		[ -z "`opkg list-installed $p`" ] && opkg install $p
	done

	# Upgrade dnsmasq to dnsmasq-full
	dnsmasqFullInstalled=`opkg list-installed dnsmasq-full`
	if [ ! "$dnsmasqFullInstalled" ] ; then
		cd /tmp
		dnsmasqFullFile=`opkg info dnsmasq-full | grep Filename | awk '{print $2}'`
		opkg download dnsmasq-full
		if [ $? == 0 -a -f "$dnsmasqFullFile" ] ; then
			opkg remove dnsmasq
			opkg install $dnsmasqFullFile
		fi
	fi

	# Upgrade wpad-mini to wpad
	wpadInstalled=`opkg list-installed wpad`
	if [ ! "$wpadInstalled" ] ; then
		cd /tmp
		wpadFile=`opkg info wpad | grep Filename | awk '{print $2}'`
		opkg download wpad
		if [ $? == 0 -a -f "$wpadFile" ] ; then
			opkg remove wpad-mini
			opkg install $wpadFile
		fi
	fi

	# Update otp package
	o1=`otp`
	o2=`CLOUD=1 otp`
	if [ "$o1" != "$o2" ] ; then
		cd /tmp
		otpFile=`opkg info otp | grep Filename | awk '{print $2}'`
		opkg download otp
		if [ $? == 0 -a -f "$otpFile" ] ; then
			opkg remove otp
			opkg install $otpFile
		fi
	fi
fi

echo "  Init sysctl"
cp $CONFIG/sysctl.conf /etc
sysctl -p

chmod +x -R $SCRIPT

echo "  Init ng_ap2"
cp $SCRIPT/ng_ap2 /etc/init.d/

if [ "$skip" != true ] ; then
	echo "  Update config"
	sh -x $SCRIPT/update/update-ng-config.sh

	/etc/init.d/ng_ap2 enable
 	/etc/init.d/ng_ap2 restart
fi

exit 0
