#!/usr/bin/env bish-bosh

ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

source $CONFIG/api.conf
source $CONFIG/mqtt.conf

source $SCRIPT/utils/date.sh
source $SCRIPT/utils/misc.sh

# subscribe topic
listenTopic="listeners/$DEVICE_ID"

# topics
statusTopic="statuses/$DEVICE_ID"
reportTopic="stats/update/$DEVICE_ID"
updateSessionTopic="sessions/update/$DEVICE_ID"
commandResponseTopic="command/response/$DEVICE_ID"

logoutFile="/tmp/tmp.logout"

# config
now=`nowAsUnitEpoch`
lastReport=$((now - 10))

bishbosh_clientId="$DEVICE_ID"
bishbosh_connect_cleanSession=1
bishbosh_connect_willTopic="$statusTopic"
bishbosh_connect_willMessage="offline"
bishbosh_connect_willQoS=1
bishbosh_connect_willRetain=1

bishbosh_connect_keepAlive=5

bishbosh_connection_handler_PINGRESP() {
  echo "ping response" >&2
}

bishbosh_connection_handler_CONNACK() {
  # send online message
  bishbosh_publishText 1 "$statusTopic" yes "online"

  # Listen server commands
  bishbosh_subscribe "$listenTopic" 1
}

bishbosh_connection_handler_PUBLISH() {
  local message=`mktemp`
  tr -d %'\r' < "$messageFilePath" > "$message"
  rm -rf "$topicNameFilePath"
  rm -rf "$messageFilePath";

  local topic="`cat $message | sed -n '1p'`"
  local data="`cat $message | sed -n '2p'`"
  rm -rf "$message"
  echo "command $topic $data" >&2
  local response=`$SCRIPT/utils/command.sh "$topic" "$data"`
  [ -z "$response" ] || bishbosh_publishText 0 "$commandResponseTopic" no "$response"
}

bishbosh_connection_handler_noControlPacketsRead() {
  now=`nowAsUnitEpoch`
  touch /tmp/now.job

  # check kick session
  if [ -s "$logoutFile" ] ; then
    echo "logout" >&2
    bishbosh_publishFileAndRemove 0 "$updateSessionTopic" no "$logoutFile"
  fi

  # update session & stats
  diff=$((now - lastReport))
  if [ "$diff" -gt 20 ] ; then
    updateData="`$SCRIPT/session/update.sh`"
    [ ! -z "$updateData" ] && bishbosh_publishText 0 "$updateSessionTopic" no "$updateData"

    updateData="`$SCRIPT/utils/report.sh`"
    if [ ! -z "$updateData" ] ; then
      echo "$updateData" >&2
      bishbosh_publishText 0 "$reportTopic" no "$updateData"
    fi

    lastReport="$now"
  fi
}

onExit() {
  echo "exit" >&2
  bishbosh_publishText 1 "$statusTopic" yes "offline"
}

# Catch onexit
trap onExit INT HUP TERM ABRT QUIT
