#!/bin/sh

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

[ -s "$CONFIG/ng.conf" ] && source $CONFIG/ng.conf
source $CONFIG/ap.conf
source $CONFIG/api.conf

source $SCRIPT/utils/date.sh
source $SCRIPT/utils/wifi.sh
source $SCRIPT/utils/network.sh
source $SCRIPT/utils/misc.sh

goOnline() {
	ssids=`getEnabledSsidList`
	radios=`getRadioCount`
	for ssid in $ssids ; do
		for radio in `seq 1 $radios` ; do
			"$SCRIPT/setup/setup-wireless-ssid.sh" "$ssid" "$radio"
		done
	done
}

enableDebug() {
	uci set wireless.debug.disabled=0
}

disableDebug() {
	uci set wireless.debug.disabled=1
}

# Disable firewall if exists
if [ ! -z "`ls -1 /etc/rc.d/ | grep firewall`" ] ; then
	/etc/init.d/firewall disable
fi

# Sometime br-wan will not be created
ifconfig br-wan &>/dev/null
if [ $? -ne 0 ] ; then
	brctl addbr br-wan
	brctl addif br-wan $WAN_ETH
	/etc/init.d/network restart
fi

# Prevent no rule or double rules
ruleCount=`iptables -t nat -vnL PREROUTING | grep HOTSPOT | wc -l`
if [ "$ruleCount" -ne 1 ] ; then
	/etc/init.d/ng_ap2 enable
	/etc/init.d/ng_ap2 restart
fi

# Restart job if needed
now=`date +%Y%m%d%H%M%S`
last30SecsInMin=$(((now - 30) / 100)) # to min
touch /tmp/now.job-ref -d "$last30SecsInMin"
isJobStillRun=`find /tmp -name now.job -newer /tmp/now.job-ref`
if [ -z "$isJobStillRun" ] ; then
	$SCRIPT/session/update.sh
	/etc/init.d/ng_ap2 restart job
fi

# Restart uhttpd if needed
isUhttpdAlive="`pgrep uhttpd`"
[ -z "$isUhttpdAlive" ] && /etc/init.d/uhttpd restart

# Schedule
schedule="`$SCRIPT/setup/setup-schedule.sh`"

# Monitor using ping
[ -z "$WAN_MONITOR_NETWORK_IP1" ] && WAN_MONITOR_NETWORK_IP1="$CLOUD"
[ -z "$WAN_MONITOR_NETWORK_IP2" ] && WAN_MONITOR_NETWORK_IP2="$DNS1"
[ -z "$WAN_MONITOR_NETWORK_IP3" ] && WAN_MONITOR_NETWORK_IP3="$DNS2"

ping1=`pingAddress "$WAN_MONITOR_NETWORK_IP1"`
ping2=`pingAddress "$WAN_MONITOR_NETWORK_IP2"`
ping3=`pingAddress "$WAN_MONITOR_NETWORK_IP3"`
pingCount=$((ping1 + ping2 + ping3))

if [ "$pingCount" == 0 ] ; then
	echo "network offline"
	status=offline
else
	echo "network online"
	status=online
fi

debugDisable=`uci get wireless.debug.disabled`
[ "$debugDisable" == 1 ] && lastStatus=online || lastStatus=offline

curHash=`md5 /etc/config/wireless`
if [ "$status" == offline ] ; then
	goOffline
	enableDebug
else
	[ "$schedule" == online ] && goOnline
	disableDebug
fi
uci commit
newHash=`md5 /etc/config/wireless`

[ "$curHash" != "$newHash" ] && wifi
if [ "$lastStatus" != "$status" ] && [ "$status" == "online" ] ; then
	/etc/init.d/ng_ap2 restart time
fi
