#!/bin/sh

[ -z "$ssid" ] && echo "missing param" 1>&2 && exit
[ -z "$sessionId" -a -z "$mac" ] && echo "missing param" 1>&2 && exit

ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

[ -f $CONFIG/ng.conf ] && source $CONFIG/ng.conf
source $CONFIG/ap.conf

source $SCRIPT/utils/wifi.sh
source $SCRIPT/utils/network.sh
source $SCRIPT/utils/hotspot.sh
source $SCRIPT/utils/qos.sh
source $SCRIPT/utils/misc.sh

baseDir="/data/hotspot${ssid}"
sessionsDir="$baseDir/sessions"
sessionsMacDir="$baseDir/sessions-mac"
[ ! -z "$mac" ] && mac=`upper $mac`

if [ ! -z "$sessionId" ] && [ -d "$sessionsDir/$sessionId" ] ; then
  sessionDir="$sessionsDir/$sessionId"
fi

if [ -z "$sessionDir" ] && [ "$force" == "self" ] && [ ! -z "$mac" ] && [ -d "$sessionsMacDir/$mac" ] ; then
  sessionDir="$sessionsMacDir/$mac"
fi

[ ! -d "$sessionDir" ] && echo "no session dir" 1>&2 && exit

cd $sessionDir
sessionId=`cat sessionId`
mac=`cat mac`

# Find ip from mac if not provided
if [ -f ipOnThisDevice ] ; then
  ip=`cat ipOnThisDevice`
else
  infName=`getSsidInf $ssid`
  ip=`getIpFromMac $mac $infName`
fi

echo "logout $mac ($ip) in ssid $ssid" 1>&2

newData=`cat totalDownload`
[ -z "$newData" ] && newData=0

if [ ! -z "$ip" ] ; then
  diffData=`getIpDataFromIpset $ssid $ip`
  [ -z "$diffData" ] && diffData=0
  newData=$((newData + diffData))

  removeIpFromIpset $ssid $ip

  if which conntrack ; then
    conntrack -D -s $ip
    conntrack -D -d $ip
  fi

  # disable qos
  disableQosOfIpOnSsid $ssid $ip
fi

if [ "$force" != "kick" ] ; then
  sessionId=`cat sessionId`
  if [ "skipWriteFile" == 1 ] ; then
    echo "logout $sessionId $newData"
  else
    echo "logout $sessionId $newData" >> /tmp/tmp.logout
  fi
fi

cd "$sessionsMacDir"
rm -rf "$mac"
cd "$sessionsDir"
rm -rf $sessionId
