#!/bin/sh

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

[ -f $CONFIG/ng.conf ] && source $CONFIG/ng.conf
source $CONFIG/ap.conf

source $SCRIPT/utils/wifi.sh
source $SCRIPT/utils/hotspot.sh
source $SCRIPT/utils/date.sh

logoutMac() {
  ssid="$1" mac="$2" force=self skipWriteFile=1 $SCRIPT/session/logout.sh
}
updateSessionOfSsid() {
  local ssid="$1"
  echo "hotspot $ssid" 1>&2

  local sessionMacDir="/data/hotspot${ssid}/sessions-mac"
  [ -d "$sessionMacDir" ] && local macs=`ls -1 "$sessionMacDir"`;

  local sessionDir="/data/hotspot${ssid}/sessions"
  [ -d "$sessionDir" ] && local sessions=`ls -1 "$sessionDir" | awk '{print "0"$1}'`;

  local sessionsData=`getAllDataFromIpset $ssid`
  local infName=`getSsidInf $ssid`
  local macList=`ip neighbor show dev $infName | grep lladdr | grep -v :: | awk '{print $1" "toupper($3)}'`

  for mac in $macs ; do
    if [ ! -d "$sessionMacDir/$mac" ] ; then
      rm -rf "$sessionMacDir/$mac"
      continue
    fi

    cd "$sessionMacDir/$mac" && echo "session $mac" 1>&2 || continue

    ip=""
    if [ -f ipOnThisDevice ] ; then
      ip=`cat ipOnThisDevice`
      echo "  found ip $ip" 1>&2
    else
      ip=`echo "$macList" | grep "$mac" | awk '{print $1}'`
      [ -z "$ip" ] && echo "  not found ip" && continue
      echo "$ip" > ipOnThisDevice
      echo "  found new ip $ip" 1>&2
    fi

    sessionId=`cat sessionId`
    sessions=`echo "$sessions" | grep -v "0${sessionId}"`

    diffData=`echo "$sessionsData" | grep "$ip " | awk '{print $2}'`
    if [ -z "$diffData" ] ; then
      echo "  no ipset, logout" 1>&2
      logoutMac $ssid $mac
      continue
    fi

    sessionsData=`echo "$sessionsData" | grep -v "$ip "`

    oldData=`cat totalDownload`
    [ -z "$oldData" ] && oldData=0

    newData=$((diffData + oldData))
    echo "  data $newData (+$diffData)" 1>&2

    newTime=`nowAsUnitEpoch`
    oldTime=`cat startedAt`
    diffTime=$((newTime - oldTime))
    echo "  time $diffTime" 1>&2

    echo -n "$newData" > totalDownload
    echo -n "$diffTime" > totalTime

    # check limitation
    if [ -s expireDate ] && [ "`cat expireDate`" -lt "$newTime" ] ; then
      echo  "  expired" 1>&2
      logoutMac $ssid $mac
      continue
    fi
    if [ -s dataRemaining ] && [ "`cat dataRemaining`" -lt "$newData" ] ; then
      echo  "  max data" 1>&2
      logoutMac $ssid $mac
      continue
    fi
    if [ -s timeRemaining ] && [ "`cat timeRemaining`" -lt "$diffTime" ] ; then
      echo  "  max time" 1>&2
      logoutMac $ssid $mac
      continue
    fi
    if [ -s sessionTimeout ] && [ "`cat sessionTimeout`" -lt "$diffTime" ] ; then
      echo  "  session timeout" 1>&2
      logoutMac $ssid $mac
      continue
    fi

    # idle
    [ -s idleTimeout ] && idleTimeout=`cat idleTimeout` || idleTimeout=900
    if [ "$diffData" -lt 3000 ] ; then
      lastActive=`cat lastActiveAt`
      idle=$((newTime - lastActive))

      if [ "$idle" -gt "$idleTimeout" ] ; then
        echo  "  idle timeout" 1>&2
        logoutMac $ssid $mac
        continue
      fi
    else
      # active
      echo -n "$newTime" > lastActiveAt
      resetTimeoutOfIpInIpset $ssid $ip $idleTimeout

      # sync session
      if [ ! -s notSync ] ; then
        sessionId=`cat sessionId`
        echo "update $sessionId $newData 0"
      fi
    fi
  done

  # remaining sessions
  for remainId in $sessions ; do
    remainId="`${remainId:1:$((${#remainId}-1))}`"
    rm -rf "$sessionDir/$remainId"
  done

  # remaining ip in set
  echo "$sessionsData" | while read -r line; do
    ip=`echo "$line" | cut -d " " -f1`
    [ ! -z "$ip" ] && removeIpFromIpset $ssid $ip
  done
}
updateSession() {
  local ssids="`getHotspotInstanceList`"

  for ssid in $ssids ; do
    updateSessionOfSsid $ssid
  done
}

echo "ng-session started" 1>&2
updateSession

exit 0