#!/bin/sh

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script
CGI=$ROOT/web/wifi
TEMPLATE=$ROOT/web/template

source $CONFIG/ap.conf
source $CONFIG/ng.conf

source $SCRIPT/utils/network.sh
source $SCRIPT/utils/wifi.sh
source $SCRIPT/utils/hotspot.sh

ssid="$1"
[ -z "$ssid" -o "$ssid" -gt "$NO_SSID" ] && echo "Invalid SSID $ssid" && exit

echo "SSID $ssid config"

eval ssidIp="\$SSID${ssid}_IP"
eval ssidDomain="\$SSID${ssid}_HOSTNAME"

echo "  uhttpd"
###########################################################
# UHTTPD

uciKey="uhttpd.ssid${ssid}"
uci -q del "$uciKey"

if [ `isHotspotEnabledOnSsid $ssid` ] ; then
  uciKey=uhttpd.hotspot
  listenPort=$((8079 + ssid))
  uci add_list "$uciKey".listen_http="${ssidIp}:${listenPort}"

  uciKey=uhttpd.autologin
  listenPort=$((7999 + ssid))
  uci add_list "$uciKey".listen_http="${ssidIp}:${listenPort}"
fi

echo "  CNA Proxy"
###########################################################
proxyConfDir="$ROOT/privoxy"
cnaBypassConf="${ssid}-bypass-cna"
injectionConf="${ssid}-injection"

rm -rf "$proxyConfDir/${cnaBypassConf}.action"
rm -rf "$proxyConfDir/${cnaBypassConf}.conf"

rm -rf "$proxyConfDir/${injectionConf}.filter"
rm -rf "$proxyConfDir/${injectionConf}.conf"

if [ `isHotspotEnabledOnSsid $ssid` ] ; then
  eval bypassCna="\$SSID${ssid}_BYPASS_CNA"
  # if [ "$bypassCna" == 1 ] ; then
    cp "$proxyConfDir/_bypass-cna.action" "$proxyConfDir/${cnaBypassConf}.action"
    cp "$proxyConfDir/_bypass-cna.conf" "$proxyConfDir/${cnaBypassConf}.conf"

    serverName="$ssidDomain"
    [ -z "$serverName" ] && serverName="$ssidIp"
    sed -i "s/__SERVER__/${serverName}/g" "$proxyConfDir/${cnaBypassConf}.action"

    echo >> "$proxyConfDir/${cnaBypassConf}.conf"
    echo "actionsfile ${cnaBypassConf}.action" >> "$proxyConfDir/${cnaBypassConf}.conf"

    cnaPort="$((8179 + ssid))"
    echo "listen-address 127.0.0.1:${cnaPort}" >> "$proxyConfDir/${cnaBypassConf}.conf"
    echo "listen-address ${ssidIp}:${cnaPort}" >> "$proxyConfDir/${cnaBypassConf}.conf"
  # fi

  templateId="`getTemplateIdOfSsid $ssid`"
  if [ ! -z "$templateId" ] && [ -s "$TEMPLATE/$templateId/_inject.html" ] ; then

    cp "$proxyConfDir/_injection.filter" "$proxyConfDir/${injectionConf}.filter"
    cp "$proxyConfDir/_injection.conf" "$proxyConfDir/${injectionConf}.conf"

    injectContent=`mktemp`
    cp "$TEMPLATE/$templateId/_inject.html" "$injectContent"
    echo >> "$injectContent"
    sed -i 's/@/\\\\\@/g' "$injectContent"
    sed -i 's/#/\\\\\#/g' "$injectContent"

    cat "$injebctContent"

    echo "s@</body>@\\n\\" >> "$proxyConfDir/${injectionConf}.filter"
    while read line; do
      echo "$line\\n\\" >> "$proxyConfDir/${injectionConf}.filter"
    done < $injectContent
    echo "\$0@g" >> "$proxyConfDir/${injectionConf}.filter"

    echo "filterfile ${injectionConf}.filter" >> "$proxyConfDir/${injectionConf}.conf"

    injectionPort="$((8279 + ssid))"
    echo "listen-address 127.0.0.1:${injectionPort}" >> "$proxyConfDir/${injectionConf}.conf"
    echo "listen-address ${ssidIp}:${injectionPort}" >> "$proxyConfDir/${injectionConf}.conf"

    rm -rf "$injectContent"
  fi
fi

echo "  DHCP"
###########################################################
# DHCP

###########################################################
# DNS - LOCAL

uci -q del dhcp.local

uciKey="dhcp.local_wifi$ssid"
uci -q del "$uciKey"

if [ `isHotspotEnabledOnSsid $ssid` ] && [ ! -z "$ssidDomain" ] ; then
  uci set "$uciKey"=domain
  uci set "$uciKey".name="$ssidDomain"
  uci set "$uciKey".ip="$ssidIp"
fi

###########################################################
# DHCP - SSID

uciKey="dhcp.wifi$ssid"
uci -q del "$uciKey"

eval dhcpEnabled="\$SSID${ssid}_DHCP"
[ -z "$DNS_DNS1" ] && DNS_DNS1="$DNS1"
[ -z "$DNS_DNS2" ] && DNS_DNS2="$DNS2"

eval dhcpDns1="\$SSID${ssid}_DHCP_DNS1"
eval dhcpDns2="\$SSID${ssid}_DHCP_DNS2"
if [ `isSsidEnabled $ssid` ] && [ ! `isSsidInBridgeMode $ssid` ] && [ "$dhcpEnabled" != 0  ] ; then
  uci set "$uciKey"=dhcp
  uci set "$uciKey".interface=wifi"$ssid"
  uci set "$uciKey".force=1

  dnsIps=""
  [ -z "$dhcpDns1" ] || dnsIps="${dnsIps},${dhcpDns1}"
  [ -z "$dhcpDns2" ] || dnsIps="${dnsIps},${dhcpDns2}"
  [ -z "$dnsIps" ] && dnsIps=",${ssidIp}"
  uci add_list "$uciKey".dhcp_option="3,${ssidIp}"
  uci add_list "$uciKey".dhcp_option="6${dnsIps}"

  eval dhcpLease="\$SSID${ssid}_DHCP_LEASETIME"
  eval dhcpLeaseUnit="\$SSID${ssid}_DHCP_LEASETIME_UNIT"
  if [ -n "$dhcpLease" -a -n "$dhcpLeaseUnit" ] ; then
    uci set "$uciKey".leasetime="${dhcpLease}${dhcpLeaseUnit}"
  fi

  if [ `isHotspotEnabledOnSsid $ssid` ] ; then
    dhcpStart=2
    dhcpLimit=250

    eval wallgarden="\$SSID${ssid}_WALLGARDEN"
    ipsetName="whitelist${ssid}"
    [ ! -z "$wallgarden" ] && wallgarden=`echo $wallgarden | tr ';' '\n'`
    for site in $wallgarden ; do
      [ -z "$site" ] && continue
      if ! [ "`addressIsIp $site`" ] ; then
        uci add_list dhcp.@dnsmasq[0].ipset="/.${site}/${site}/${ipsetName}"
      fi
    done
  else
    eval dhcpStart="\$SSID${ssid}_DHCP_START"
    eval dhcpLimit="\$SSID${ssid}_DHCP_LIMIT"
    [ -z "$dhcpStart" ] && dhcpStart=2
    [ -z "$dhcpLimit" ] && dhcpLimit=250
  fi
  uci set "$uciKey".start="$dhcpStart"
  uci set "$uciKey".limit="$dhcpLimit"
fi

echo "  Network"
###########################################################

uciKey="network.wifi${ssid}"
uci -q del "$uciKey"

if [ `isSsidEnabled $ssid` ] && [ ! `isSsidInBridgeMode $ssid` ] ; then
  ifname=`getSsidInf $ssid`
  eval netmask="\$SSID${ssid}_NETMASK"
  [ "$netmask" ] || netmask="255.255.255.0"

  uci set "$uciKey"=interface
  uci set "$uciKey".type=bridge
  uci set "$uciKey".ifname="$ifname"
  uci set "$uciKey".proto=static
  uci set "$uciKey".ipaddr="$ssidIp"
  uci set "$uciKey".netmask="$netmask"
fi
