#!/bin/sh

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

[ -f $CONFIG/ng.conf ] && source $CONFIG/ng.conf
source $CONFIG/ap.conf
source $CONFIG/api.conf
source $SCRIPT/utils/token.sh
source $SCRIPT/utils/network.sh
source $SCRIPT/utils/misc.sh

[ -z "$1" ] && sleep 2

echo "Updating ng config..."
configFile=`mktemp`

wget -vt 3 -T 5 $(getAuthHeader) -O "$configFile" "$CLOUD_ADDRESS/ng.conf"

if [ -s "$configFile" ] ; then
  validConfig=`cat $configFile | grep HASH_CONFIG_SITE`
  if [ ! -z "$validConfig" ] ; then
    curConfHash=`md5 /www/config/ng.conf`
    newConfHash=`md5 "$configFile"`
    if [ "$curConfHash" != "$newConfHash" ] ; then
      echo "  applying new config"
      mv "$configFile" $CONFIG/ng.conf
      [ "$skip" != true ] && apply=true
    else
      echo "  config is up-to-date"
    fi
  else
    echo "  config is invalid"
  fi
else
  echo "  config is up-to-date"
fi
rm -rf "$configFile"

echo "Updating mqtt config..."
mac=`getWanMac`
wget -qt 3 -T 5 $(getAuthHeader) -O "$configFile" "$CLOUD_ADDRESS/mqtt.conf?mac=$mac"

if [ -s "$configFile" ] ; then
  validConfig=`cat $configFile | grep bishbosh_connect_username`
  if [ $? -eq 0 ] ; then
    hashMqttOld=`md5 /www/config/mqtt.conf`
    hashMqttNew=`md5 "$configFile"`
    if [ "$hashMqttOld" != "$hashMqttNew" ] ; then
      mv "$configFile" $CONFIG/mqtt.conf
      /etc/init.d/ng_ap2 restart job
    fi
  else
    echo "  mqtt config is invalid"
  fi
fi
rm -rf "$configFile"

# Get new template before apply config
$SCRIPT/update/update-template.sh

[ "$apply" ] && $SCRIPT/update/apply-config.sh

exit 0
