#!/bin/sh

[ -z "$ROOT" ] && ROOT=/www
CONFIG=$ROOT/config
SCRIPT=$ROOT/script

getHotspotInstanceList() {
  local list=""
  for instance in `seq 1 "$NO_SSID"` ; do
    eval local enabled="\$SSID${instance}_ENABLED"
    eval local hotspotEnabled="\$SSID${instance}_HOTSPOT"
    [ "$enabled" == 1 -a "$hotspotEnabled" == 1 ] && list="$list $instance";
  done

  echo $list
}

isHotspotEnabledOnSsid() {
  local instance="$1"

  eval local enabled="\$SSID${instance}_ENABLED"
  eval local hotspotEnabled="\$SSID${instance}_HOTSPOT"
  [ "$enabled" == 1 -a "$hotspotEnabled" == 1 ] && echo 1
}

getAllDataFromIpset() {
  ipset list -! loginlist$1 | grep [.] | awk '{print $1" "$7}'
}

getIpDataFromIpset() {
  ipset list -! loginlist$1 | grep "$2 " | awk '{print $7}'
}

addIpToIpset() {
  ipset add -! loginlist$1 $2 timeout 3600 bytes 0
  ipset del -! bypass$1 $2 2>/dev/null
}

removeIpFromIpset() {
  ipset del -! loginlist$1 $2
  ipset del -! bypass$1 $2 2>/dev/null
}

clearIpset() {
  ipset flush -! loginlist$1
}

resetTimeoutOfIpInIpset() {
  ipset add -! loginlist$1 $2 timeout $3 bytes 0
}

getTemplateIdOfSsid() {
  local instance="$1"

  [ ! -s "$CONFIG/template.conf" ] && exit
  source "$CONFIG/template.conf"

  eval echo "\$TEMPLATE_SSID${instance}"
}
