#!/bin/sh

# Network helpers
#################

getWanMac() {
	ifconfig $WAN_ETH | grep HWaddr | awk '{print $5}'
}
getWanIp() {
	[ -z "$WAN_VLAN" ] && wanInf=br-wan || wanInf="br-vlan${WAN_VLAN}"
	ifconfig $wanInf | grep "inet addr" | awk '{print $2}' | awk -F ':' '{printf $2}'
}

getLanMac() {
	ifconfig $LAN_ETH | grep HWaddr | awk '{print $5}'
}

getVlanInf() {
	local vlan="$1"
	local wan=`echo $WAN_ETH | awk -F '.' '{printf $1}'`
	echo "$wan.$vlan"
}

# Get default gateway IP
getGateway() {
	route | grep default | awk '{printf $2}'
}
pingAddress() {
	ping -c 3 -w 3 $1 2>/dev/null | grep 'received' | awk '{ print $4 }'
}

getIpFromMac() {
	local mac=`echo "$1" | tr [A-Z] [a-z]`

	if [ ! -z "$2" ] ; then
		ip neighbor show dev $2 | grep "$mac" | awk '{print $1}'
	else
		ip neighbor show | grep "$mac" | awk '{print $1}'
	fi
}

resolveHostname() {
	host=$1
	nslookup $1 2>/dev/null | sed 's/[^0-9. ]//g' | tail -n 1 | awk -F " " '{print $2}'
}

getNetworkReport() {
	echo "ip `getWanIp`"
  echo "mac `getWanMac`"
  echo "gateway `getGateway`"
}

addressIsIp() {
	ip="$1"

	if expr "$ip" : '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$' >/dev/null; then
		echo "1"
	else
		echo ""
	fi
}