#!/bin/sh

isQosEnabledOnSsid() {
  local ssid="$1"

  eval local enabled="\$SSID${ssid}_ENABLED"
  eval local qosEnabled="\$SSID${ssid}_QOS"
  [ "$enabled" == 1 -a "$qosEnabled" == 1 ] && echo 1
}

isQosActivatedOnSsid() {
  local ssid="$1"
  local inf="br-wifi${ssid}"

  qdisc=`tc qdisc show dev $inf 2>/dev/null | grep hfsc`
  [ ! -z "$qdisc" ] && echo 1
}

# enable & disable
disableQosOnSsid() {
  local ssid="$1"
  local inf="br-wifi${ssid}"

  tc qdisc    del dev $inf  root &>/dev/null
}

enableQosOnSsid() {
  local ssid="$1"
  disableQosOnSsid $1

  local inf="br-wifi${ssid}"

  tc qdisc    add dev $inf  root           handle 1 hfsc default 1
  tc class    add dev $inf  parent 1:      classid 1:1 hfsc sc rate 100Mbit ul rate 100Mbit
}
enableQosOnSsid() {
  local ssid="$1"
  if [ `isSsidInBridgeMode $ssid` ] ; then
    local infs=`getSsidWirelessInf $ssid`
    for inf in $infs ; do
      enableQosOnInf $inf
    done
  else
    enableQosOnInf "br-wifi${ssid}"
  fi
}

# enable ip & disable ip
getQosIndexOfIp() {
  local ip="$1"
  local ip3=`echo "$ip" | awk -F'.' '{print $3}'`
  local ip4=`echo "$ip" | awk -F'.' '{print $4}'`
  local index=$((ip3 * 256 + ip4))

  printf '%x' $index
}

disableQosOfIpOnSsid() {
  local ssid="$1"
  local inf="br-wifi${ssid}"

  [ ! `isQosActivatedOnSsid "$ssid"` ] && return

  local ip="$2"
  local index=`getQosIndexOfIp "$ip"`

  pref=`tc filter show dev $inf | grep flowid | grep "1:$index" | awk '{print $7}'`
  [ ! -z "$pref" ] && tc filter del   dev $inf  pref "$pref" &>/dev/null
  tc qdisc  del   dev $inf  parent 1:"$index" &>/dev/null
  tc class  del   dev $inf  classid 1:"$index" &>/dev/null
}

enableQosOfIpOnSsid() {
  local ssid="$1"
  local inf="br-wifi${ssid}"

  [ ! `isQosActivatedOnSsid "$ssid"`  ] && enableQosOnSsid $ssid

  local ip="$2"
  local speed="$3"
  [ -z "$speed" -o "$speed" == 0 ] && return

  disableQosOfIpOnSsid "$ssid" "$ip"

  local unit="Mbit"
  [ "$speed" -gt 1000 ] && unit="Kbit"
  [ "$speed" -gt 1000000 ] && unit="bit"
  local index=`getQosIndexOfIp "$ip"`

  tc class  add   dev $inf  parent 1: classid 1:"$index" hfsc sc rate "${speed}${unit}" ul rate "${speed}${unit}"
  tc qdisc  add   dev $inf  parent 1:"$index" handle "$index": fq_codel
  tc filter add   dev $inf  parent 1: protocol all u32 match ip dst "$ip"/32 flowid 1:"$index"
}