JSON = (loadfile "/www/web/wifi/JSON.lua")()

---------------------
-- GLOBAL VARIABLE --

local errorMessages = {
  system = "Lỗi hệ thống. Vui lòng liên lạc quản trị viên",
  credential = "Thông tin đăng nhập không hợp lệ"
}

----------------------
-- LIBRARY FUNCTION --

local function getHost(env)
  return env.HTTP_HOST or env.headers["Host"] or env.headers["host"]
end

---------------------
-- RENDER TEMPLATE --

local function getTemplateId(env)
  local ssid = getSsid(env)

  dofile "/www/config/template.conf"
  return _G["TEMPLATE_SSID" .. ssid] or "default"
end

local function getTemplateFile(env, fileName)
  local templateId = getTemplateId(env)

  local tplFile = "/www/web/template/" .. templateId .. "/" .. fileName
  if not fileExists(tplFile) then
    tplFile = "/www/web/template/default/" .. fileName
  end

  return tplFile
end

local baseFragmentFile = "/www/web/template/default/base.html"
local function renderFragment(env, fileName, params)
  local bodyFile = getTemplateFile(env, fileName)
  local body = table.concat(readFileAsText(bodyFile))

  params = (params or {})
  if not params.title then
    params.title = "MegaNet"
  end
  if not params.url then
    params.url = getRedirectUrl(env)
  end
  params.body = body

  log({render = fileName, url = params.url})
  renderHtml(baseFragmentFile, params)
end

local function renderLoginPage(env, error)
  local ip = env.REMOTE_ADDR
  local mac = getMac(env)

  local tplFile = getTemplateFile(env, "index.html")
  log({ip = ip, message = "render login", file = tplFile})

  if not error then
    error = ""
  end

  renderHtml(tplFile, {mac = mac, error = error})
end

local function renderCnaBypass(env, mode)
  local ip = env.REMOTE_ADDR
  local host = getServerHost(env)

  local tplFile = "/www/web/template/default/bypass.html"
  renderHtml(tplFile, {mode = mode, host = host})
end

local function renderPoweredBy(env)
  local ssid = getSsid(env)
  local serverHost = getServerHost(env)
  local url = "http://" .. serverHost .. "/wifi/pre?src=powered"

  if (_G["SSID" .. ssid .. "_HIDE_POWERED"] == "1") then
    redirect(url)
  else
    renderFragment(
      env,
      "_powered.html",
      {title = "Powered by MegaNet", url = url}
    )
  end
end

--------------------
-- BYPASS PROCESS --

function bypassCna(env)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)

  local cmd = "ipset add -! bypass" .. ssid .. " " .. ip
  pcall(cmdCall, cmd)

  renderCnaBypass(env, "afterBypass")
end

--------------------
-- LOGOUT PROCESS --

function logout(env, not_a_request)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)
  local mac = getMac(env)
  if not mac then
    return redirect("/wifi/logout")
  end

  log({ip = ip, message = "logout"})
  local logoutData = {
    ip = ip,
    mac = mac,
    ssid = ssid,
    force = "self"
  }

  local cmd =
    table2Qs(logoutData, " ", "=", true) .. " /www/script/session/logout.sh"

  pcall(cmdCall, cmd)
  sleep(1)

  if not not_a_request then
    renderHtml("/www/web/template/default/logout.html", {ip = ip, mac = mac})
  end
end

-------------------
-- LOGIN PROCESS --

local function bypass(env, res)
  if not isLoggedIn(env) then
    _bypass(env, res)
  end

  local url = res.redirect or getRedirectUrl(env)
  renderFragment(env, "_success.html", {title = "MegaNet", url = url})
end

local function callLogin(env)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)
  local mac = getMac(env)

  local queryData =
    table2Qs(
    {
      ip = ip,
      mac = mac,
      ssid = ssid,
      ua = env.HTTP_USER_AGENT
    }
  )

  local url = _G["AUTH_ADDRESS"] .. "/login?" .. queryData
  local requestData = {
    url = url,
    headers = getAuthHeader()
  }

  local status, code, res = pcall(requestCall, requestData)
  log(code)
  if status == false or res == "" or not res then
    -- Request send failed, may be cause by network
    res = {
      error = false,
      session = {
        mac = mac,
        startedAt = nowAsISOString()
      }
    }
    log("call login failed")
  else
    res = JSON:decode(res)
    log(res)
  end

  return res
end

local function callVerify(env, body)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)
  local mac = getMac(env)

  -- Call server verify
  local verifyData = {
    ip = ip,
    mac = mac,
    ssid = ssid,
    ua = env.HTTP_USER_AGENT
  }
  if body.code ~= nil and body.code ~= "" then
    verifyData.code = body.code
  else
    verifyData.username = body.username
    verifyData.password = body.password
  end

  local url = _G["AUTH_ADDRESS"] .. "/verify?" .. table2Qs(verifyData)
  local requestData = {
    url = url,
    headers = getAuthHeader()
  }

  local status, code, res = pcall(requestCall, requestData)
  log(code)

  if status == false or not res or res == "" then
    -- Request send failed, may be cause by network
    log("call verify failed")
    res = {error = true, message = errorMessages.system}
  else
    res = JSON:decode(res)
    log(res)
  end

  return res
end

local function loginFree(env, body)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)
  log({ip = ip, message = "login free"})

  if not body then
    body = {}
  end

  -- Call server verify
  local res = callLogin(env)

  -- Accept only when status code is 200
  if res.error == true or res.error == "true" or not res.session then
    return renderLoginPage(env, res.message)
  end

  if body.redirect and body.redirect ~= "" then
    res.redirect = body.redirect
  end

  local limitations = res.limitations or {}
  if body.maxUpload ~= nil then
    limitations.maxUpload = body.maxUpload
  end
  if body.maxDownload ~= nil then
    limitations.maxDownload = body.maxDownload
  end
  if body.sessionTimeout ~= nil then
    limitations.sessionTimeout = body.sessionTimeout
  end
  res.limitations = limitations

  bypass(env, res)
end

local function loginBilling(env, body)
  local ip = env.REMOTE_ADDR
  log({ip = ip, message = "verify"})

  if not body then
    return renderLoginPage(env, errorMessages.credential)
  end
  if body.code == "" then
    return renderLoginPage(env, errorMessages.credential)
  end
  if not body.code and (not body.username or body.username == "") then
    return renderLoginPage(env, errorMessages.credential)
  end
  if not body.code and (not body.password or body.password == "") then
    return renderLoginPage(env, errorMessages.credential)
  end

  local res = callVerify(env, body)

  -- Accept only when status code is 200
  if res.error == true or res.error == "true" then
    return renderLoginPage(env, res.message)
  end

  bypass(env, res)
end

function login(env, not_a_request)
  local ssid = getSsid(env)
  local ip = env.REMOTE_ADDR

  if isLoggedIn(env) then
    log({ip = ip, message = "already logged in"})
    return handleLoggedIn(env)
  end

  local mac = getMac(env)
  if not mac or mac == "" then
    return renderLoginPage(env)
  end

  local post_length = env.CONTENT_LENGTH or 0
  local rawBody = ""
  local body = {}

  if env.REQUEST_METHOD == "POST" and tonumber(post_length) > 0 then
    local n = tonumber(env.CONTENT_LENGTH)
    rawBody = io.read(n)
    body = parseQueryString(rawBody)
  elseif env.REQUEST_METHOD == "GET" then
    rawBody = env.QUERY_STRING
    body = parseQueryString(env.QUERY_STRING)
  end

  if isAuthenticationRequired(env) then
    return loginBilling(env, body)
  else
    return loginFree(env, body)
  end
end

------------------
-- SESSION INFO --

function sessionInfo(env)
  local ip = env.REMOTE_ADDR
  local mac = getMac(env)
  local logged = isLoggedIn(env)

  response(JSON:encode({ip = ip, mac = mac, logged = logged}), {json = true})
end

-------------------
-- FIRST REQUEST --

function preLogin(env)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)

  -- local request
  if (ip == "127.0.0.1") then
    return renderLoginPage(env)
  end

  -- already logged in
  if isLoggedIn(env) then
    log({ip = ip, message = "already logged in"})
    return handleLoggedIn(env, true)
  end

  -- check current session
  if isAutoLoginEnabled(env) then
    local res = callGetCurrentSession(env)
    if res.session ~= nil then
      log({ip = ip, message = "has current session"})
      _bypass(env, res)

      log({ip = ip, message = "redirect"})
      local url = getRedirectUrl(env)
      if not isCnaRequest(env) then
        url = (env.headers["referer"] or env.headers["Referer"] or url)
      end
      return redirect(url)
    end
  end

  local host = getHost(env)
  local serverHost = getServerHost(env)
  log({host = host, serverHost = serverHost})

  -- first request
  if not string.find(host, serverHost) then
    -- show powered
    log({ip = ip, message = "render powered by"})
    return renderPoweredBy(env)
  end

  if
    _G["SSID" .. ssid .. "_BYPASS_CNA"] == "1" and isCnaRequest(env) and
      not isCnaRequestBypassed(env)
   then
    log({ip = ip, message = "render cna bypass"})
    return renderCnaBypass(env, "bypass")
  end

  -- check after bypass
  local query = parseQueryString(env.QUERY_STRING)
  if query.src == "bypass" and isCnaRequest(env) then
    log({ip = ip, message = "cannot escape cna"})
    return response("Escape failed!")
  end

  log({ip = ip, message = "render login"})
  return renderLoginPage(env)
end
